import express from 'express'
import fs from 'fs'
import fse from 'fs-extra'
import mkdirp from 'mkdirp'
import {minify} from 'html-minifier'
import extend from 'extend'
import User from '../../cli/controllers/User'
import * as abe from '../../cli'

import {
  fileAttr,
  save,
  getAttr, getEnclosingTags, escapeTextToRegex,
  Util,
  FileParser,
  fileUtils,
  config,
  cli,
  log,
  Page,
  Locales,
  getTemplate,
  Hooks,
  Plugins,
  serveSite
} from '../../cli'

import {editor} from './editor'
import locale from '../helpers/abe-locale'

var router = express.Router()

// Abe templating engine route
router.get('/abe/*', function(req, res, next) {
  Hooks.instance.trigger('beforeRoute', req, res, next)
  if(typeof res._header !== 'undefined' && res._header !== null) return;

  var templatePath = fileUtils.getTemplatePath(req.params[0])
  var filePath = fileUtils.getFilePath(req.query.filePath)
  var obj
  var tplUrl
  var isHome = true
  var manager = {}
  var user = new User(req.session, req.query)

  var p = new Promise((resolve, reject) => {

    if(templatePath !== null && filePath !== null) {

      isHome = false

      if(!fileAttr.test(filePath)){
        var folderFilePath = filePath.split('/')
        folderFilePath.pop()
        folderFilePath = fileUtils.pathWithRoot(folderFilePath.join('/'))
        mkdirp.sync(folderFilePath)
        var files = FileParser.getFiles(folderFilePath, true, 2)
        var latest = fileAttr.filterLatestVersion(fileAttr.getFilesRevision(files, filePath), 'draft')
        if(latest.length) {
          filePath = latest[0].path
        }
      }

      tplUrl = FileParser.getFileDataFromUrl(filePath)
      var fakeContent = (req.query.fakeContent) ? true : false

      var p2 = new Promise((resolve2, reject2) => {
        if(!fileUtils.isFile(tplUrl.json.path)) {
          var json = {}
          var tpl = templatePath
          var text = getTemplate(tpl)
          text = Util.removeDataList(text)

          save(filePath, req.params[0], json, text, 'draft', null, 'draft', user)
            .then((resSave) => {
                filePath = resSave.htmlPath
                tplUrl = FileParser.getFileDataFromUrl(filePath)
                resolve2()
              }).catch(function(e) {
                console.error(e.stack)
              })
        }else {
          resolve2()
        }
      })

      p2.then(() => {

        editor(templatePath, tplUrl, fakeContent)
          .then((res) => {
            obj = res

            FileParser.getAssetsFolder()
            FileParser.getAssets() 

            var revisionFilePath = FileParser.changePathEnv(filePath, config.draft.url)
            var dirPath = fileUtils.removeLast(revisionFilePath)
            var allDraft = FileParser.getFiles(dirPath, true, 99, new RegExp("\\." + config.files.templates.extension))

            allDraft = FileParser.getMetas(allDraft, 'draft')
            var breadcrumb = req.params[0].split('/')
            manager.file = {
              revision: fileAttr.getFilesRevision(allDraft, fileAttr.delete(revisionFilePath))
              ,template: breadcrumb
              ,path: (req.query.filePath) ? fileUtils.cleanTplName(req.query.filePath) : ''
            }
            if(manager.file.revision.length > 0){
              var publishPath = fileAttr.delete(manager.file.revision[0].path.replace(new RegExp(`/${config.draft.url}/`), `/${config.publish.url}/`))
              manager.file.isPublished = fileUtils.isFile(publishPath)
            }
            resolve()
          }).catch(function(e) {
            console.error(e.stack)
          })
        }).catch(function(e) {
          console.error(e.stack)
        })
    }else {
      resolve()
    }
  }).catch(function(e) {
    console.error(e.stack) // "oh, no!"
  })

  p.then(() => {

    manager.home = {
      files: FileParser.getAllFiles()
    }

    manager.list = FileParser.getProjetFiles()
    manager.editConfig = config.getConfigByWebsite()
    manager.config = JSON.stringify(config)
    
    var _hasBlock = (obj) ? obj.hasBlock : false
    var _hasSingleBlock = (obj) ? obj.hasSingleBlock : false
    var _template = (filePath) ? '/page/' + req.params[0] + `?filePath=${req.query.filePath}` : false
    var _form = (obj) ? obj.form : false
    var _json = (obj) ? obj.json : false
    var _file = (tplUrl) ? tplUrl.draft.file : false
    var _filePath = (req.query.filePath) ? req.query.filePath : false

    // var textMinified = minify(text.replace(/'/g, "\\'"), {collapseWhitespace: true})
    res.render(config.abeEngine, { // render html and send ABE engine to users
      abeUrl: '/abe/',
      test: JSON.stringify(locale),
      text: locale,
      templatePath: req.params[0],
      template: _template,
      hasSingleBlock: _hasSingleBlock,
      hasBlock: _hasBlock,
      form: _form,
      urlToSaveFile: _filePath,
      tplName: _file,
      json: _json,
      config: config,
      Locales: Locales.instance.i18n,
      manager: manager,
      allowed: {
        action: user.actions,
        workflow: user.workflow,
        index: user.workflowIndex
      }
    })
  })
})

function page(req, res, next) {
  var templatePath = fileUtils.getTemplatePath(req.params[0])
  var filePath = fileUtils.getFilePath(req.query.filePath)
  var html = (req.query.html) ? true : false
  var json = null
  if(typeof req.body.json !== 'undefined' && req.body.json !== null) {
    if(typeof req.body.json === 'string') {
      json = JSON.parse(req.body.json)
    }else {
      json = req.body.json
    }
  }

  if(typeof filePath !== 'undefined' && filePath !== null) {

    if(!fileAttr.test(filePath)) {
      var folderFilePath = filePath.split('/')
      folderFilePath.pop()
      folderFilePath = fileUtils.pathWithRoot(folderFilePath.join('/'))
      mkdirp.sync(folderFilePath)
      var files = FileParser.getFiles(folderFilePath, true, 2)
      var latest = fileAttr.filterLatestVersion(fileAttr.getFilesRevision(files, filePath), 'draft')
      if(latest.length) {
        filePath = latest[0].path
      }
    }

    var tplUrl = FileParser.getFileDataFromUrl(filePath)
    
    log.write('Index-route-page', 'templatePath', templatePath, 'tplUrl', tplUrl)

    if(typeof json === 'undefined' || json === null) {
      json = FileParser.getJson(tplUrl.json.path)
    }
    
    let meta = config.meta.name
    let extension = config.files.templates.extension

    var template = ''
    if(typeof json[meta] !== 'undefined' && json[meta] !== null && json[meta] !== ''
      && json[meta].template !== 'undefined' && json[meta].template !== null && json[meta].template !== '') {
      template = json[meta].template
    }else {
      template = fileUtils.getTemplatePath(req.params[0])
    }
    var text = getTemplate(template)

    Util.getDataList(fileUtils.removeLast(tplUrl.publish.link), text, json)
      .then(() => {
        var page = new Page(templatePath, text, json, html)
        res.set('Content-Type', 'text/html')
        res.send(page.html)
      }).catch(function(e) {
        console.error(e.stack)
      })
  }else {
    // not 404 page if tag abe image upload into each block
    if(/upload%20image/g.test(req.url)) {
      var b64str = 'data:image/gif;base64,R0lGODlhAQABAAAAACH5BAEKAAEALAAAAAABAAEAAAICTAEAOw=='
      var img = new Buffer(b64str, 'base64');

      res.writeHead(200, {
        'Content-Type': 'image/jpeg',
        'Content-Length': img.length
      });
      res.end(img); 
    }else {
      log.write('Index-route-page', '404 not found', 'filePath', filePath)
      res.status(404).send('Not found');
    }
  }
}

// Template page route
router.post('/page/*', function(req, res, next) {
  Hooks.instance.trigger('beforeRoute', req, res, next)
  if(typeof res._header !== 'undefined' && res._header !== null) return;

  page(req, res, next)
})

// Template page route
router.get('/page/*', function(req, res, next) {
  Hooks.instance.trigger('beforeRoute', req, res, next)
  if(typeof res._header !== 'undefined' && res._header !== null) return;

  page(req, res, next)
})

router.post('/save', function(req, res, next){
  Hooks.instance.trigger('beforeRoute', req, res, next)
  if(typeof res._header !== 'undefined' && res._header !== null) return;

  var saveAction = req.body.saveAction
  var user = new User(req.session, req.query)
  var p = new Promise((resolve, reject) => {
    if(saveAction === 'publish'){
      save(
        fileUtils.getFilePath(req.body.filePath),
        req.body.tplPath,
        req.body.json,
        '',
        'draft',
        null,
        saveAction,
        user)
        .then(() => {
          resolve()
        }).catch(function(e) {
          console.error(e.stack)
        })
    }else if(saveAction !== 'draft'){
      save(
        fileUtils.getFilePath(req.body.filePath),
        req.body.tplPath,
        req.body.json,
        '',
        'draft',
        null,
        saveAction,
        user)
        .then((resSave) => {
          resolve(resSave)
        }).catch(function(e) {
          console.error(e.stack)
        })
    }
    else {
      resolve()
    }
  })

  p.then((resSave) => {
    save(
      fileUtils.getFilePath(req.body.filePath),
      req.body.tplPath,
      req.body.json,
      '',
      saveAction,
      resSave,
      saveAction,
      user)
      .then((resSave) => {
        if(typeof resSave.error !== 'undefined' && resSave.error !== null  ){
          res.set('Content-Type', 'application/json')
          res.send(JSON.stringify({error: resSave.error}))
        }
        var result
        if(typeof resSave.reject !== 'undefined' && resSave.reject !== null){
          result = resSave
        }
        if(typeof resSave.json !== 'undefined' && resSave.json !== null){
          result = {
            success: 1,
            json: resSave.json
          }
        }
        res.set('Content-Type', 'application/json')
        res.send(JSON.stringify(result))
      })
  }).catch(function(e) {
    console.error(e.stack)
  })
})

router.get('/save-config', function(req, res, next){
  Hooks.instance.trigger('beforeRoute', req, res, next)
  if(typeof res._header !== 'undefined' && res._header !== null) return;

  config.save(req.query.website, req.query.json)
  res.set('Content-Type', 'application/json')
  res.send(JSON.stringify(req.query.json))
})

router.get('/unpublish', function(req, res, next){
  Hooks.instance.trigger('beforeRoute', req, res, next)
  if(typeof res._header !== 'undefined' && res._header !== null) return;

  var filePath = req.query.filePath
  var dirPath = FileParser.unpublishFile(filePath)

  var result = {
    success: 1,
    file: req.query.filePath
  }
  res.set('Content-Type', 'application/json')
  res.send(JSON.stringify(result))
})

router.get('/delete', function(req, res, next){
  Hooks.instance.trigger('beforeRoute', req, res, next)
  if(typeof res._header !== 'undefined' && res._header !== null) return;

  var filePath = req.query.filePath
  var dirPath = FileParser.deleteFile(filePath)

  var result = {
    success: 1,
    file: req.query.filePath
  }
  res.set('Content-Type', 'application/json')
  res.send(JSON.stringify(result))
})

router.get('/', function(req, res, next){
  Hooks.instance.trigger('beforeRoute', req, res, next)
  if(typeof res._header !== 'undefined' && res._header !== null) return;


  res.redirect('/abe/');
})

router.post('/upload/*', function(req, res, next){
  Hooks.instance.trigger('beforeRoute', req, res, next)
  if(typeof res._header !== 'undefined' && res._header !== null) return;

  var resp = {success: 1}
  var filePath
  var fstream
  var folderWebPath = '/' + config.upload.image
  folderWebPath = Hooks.instance.trigger('beforeSaveImage', folderWebPath, req.headers.referer)
  var folderFilePath = fileUtils.concatPath(config.root, config.publish.url, folderWebPath)
  mkdirp.sync(folderFilePath)

  req.pipe(req.busboy)
  req.busboy.on('file', function (fieldname, file, filename) {
    filePath = fileUtils.concatPath(folderFilePath, filename)
    resp['filePath'] = fileUtils.concatPath(folderWebPath, filename)
    fstream = fs.createWriteStream(filePath)
    file.pipe(fstream)
    fstream.on('close', function () {});
  });
  req.busboy.on('finish', function() {
    res.set('Content-Type', 'application/json')
    res.send(JSON.stringify(resp))
  });
})

var site = new serveSite()

router.get('/site*', function(req, res, next) {
  Hooks.instance.trigger('beforeRoute', req, res, next)
  if(typeof res._header !== 'undefined' && res._header !== null) return;

  if(!site.isStarted) site.start(config)
  res.set('Content-Type', 'application/json')
  res.send(JSON.stringify(site.infos))
})

var routes = Plugins.instance.getRoutes()
Array.prototype.forEach.call(routes, (route) => {
  if(typeof route.get !== 'undefined' && route.get !== null) {
    Array.prototype.forEach.call(route.get, (routeGet) => {
      try {
        var pluginRoot = require(routeGet.path)
        if(typeof pluginRoot.default === 'object') {
          router.get(routeGet.routePath, pluginRoot.default)
        }else {
          router.get(routeGet.routePath, function (req, res, next) {
            pluginRoot.default(req, res, next, abe)
          })
        }
      } catch(e) {
        // statements
        console.log(e);
      }
    })
  }
  if(typeof route.post !== 'undefined' && route.post !== null) {
    Array.prototype.forEach.call(route.post, (routePost) => {
      try {
        var pluginRoot = require(routePost.path)
        if(typeof pluginRoot.default === 'object') {
          router.post(routePost.routePath, pluginRoot.default)
        }else {
          router.post(routePost.routePath, function (req, res, next) {
            pluginRoot.default(req, res, next, abe)
          })
        }
      } catch(e) {
        // statements
        console.log(e);
      }
    })
  }
})

export default router
