import EditorUtils from '../modules/EditorUtils'
import EditorInputs from '../modules/EditorInputs'
import {IframeNode} from '../utils/iframe'
import Nanoajax from 'nanoajax'
import qs from 'qs'
import {Promise} from 'es6-promise'

export default class EditorFiles {
  constructor() {
    this._ajax = Nanoajax.ajax
    this._filePathEle = document.getElementById('file-path')

    this._handleChangeFiles = this._changeFiles.bind(this)
  	this.rebind()
  }

  rebind() {
    let files = [].slice.call(document.querySelectorAll('input[type="file"]'))

    Array.prototype.forEach.call(files, (file) => {
      file.removeEventListener('change', this._handleChangeFiles)
      file.addEventListener('change', this._handleChangeFiles)
    })
  }

  _changeFiles(e) {
    this._uploadFile(e.target)
  }

  _uploadFile(target){
    var formData = new FormData()
    if (target.value == '') {
      console.log("Please choose file!")
      return false
    }
    EditorUtils.scrollToInputElement(target)
    var parentTarget = target.parentNode.parentNode
    var percent = parentTarget.querySelector('.percent')
    var file = target.files[0]

    formData.append('uploadfile', file)
    var xhr = new XMLHttpRequest()
    xhr.open('post', '/upload/' + '?baseUrl=' + CONFIG.FILEPATH, true)
    xhr.upload.onprogress = (e) => {
      if (e.lengthComputable) {
        var percentage = (e.loaded / e.total) * 100
        percent.textContent = percentage.toFixed(0) + '%'
      }
    }
    xhr.onerror = (e) => { console.log('An error occurred while submitting the form. Maybe your file is too big') }
    xhr.onload = () => {
      var resp = JSON.parse(xhr.responseText)
      var input = parentTarget.querySelector('input.image-input')
      input.value = resp.filePath
      var nodes = IframeNode('#page-template', '[data-abe-' + input.id + ']')
      Array.prototype.forEach.call(nodes, (node) => {
        EditorUtils.formToHtml(node, input)
      })
      EditorInputs._inputChanged()
    }
    percent.textContent = '0%'
    xhr.send(formData)
  }
}